## An Introduction to Biostatistical Analysis with R
## R codes for lecture on 04/07/2017

## 1st step: Begin with the ABCs of R

# simple calculation
3 + 5 * 3

# assign value to a varible
x <- 1 + 2
x

# calculation with the value assigned variable
x + 5 * x

# calculation with various functions
abs(x)
sin(x)
atan(x)
log(x)
log10(x)

# calculate PDF of normal distribution with mean mu and sd sigma
mu <- 3
s2 <- 2
x <- 5
1 / sqrt(2 * pi * s2) * exp(- (x - mu)^2 / (2 * s2)) 

# calculate PDF of normal distribution using the R fanction dnorm
dnorm(x, mu, sqrt(s2))

# vectorized arithmetic
length <- c(8.1, 7.7, 8.2, 9.7, 7.1, 7.3)    # mm scale
length

# calculate length-width ratio
width <- c(3.7, 3.0, 2.9, 2.4, 3.3, 2.5)
ratio <- length / width
ratio

# calculate mean of seed length
sum(ratio)
length(ratio)
sum(ratio) / length(ratio)

# calculate mean using the R function mean
mean(ratio)

# calculate variance
xbar <- mean(ratio)
(ratio - xbar)^2
sum((ratio - xbar)^2)
sum((ratio - xbar)^2) / (length(ratio) - 1)

# calculate standard deviation using the R function sd
var(ratio)

# calculate covariance
xbar <- mean(length)
ybar <- mean(width)
sum((length - xbar) * (width - ybar)) / (length(length) - 1)

# calculate covariance using the R function cov
cov(length, width)

# calculate correlation
s12 <- sum((length - xbar) * (width - ybar))
s1 <- sum((length - xbar)^2)
s2 <- sum((width - ybar)^2)
s12 / (sqrt(s1) * sqrt(s2))

# calculate correlation with cov and sd
cov(length, width) / (sd(length) * sd(width))

# calculate correlation using the R function cor
cor(length, width)

# combine two vectors to make a 6x2 matrix
x <- cbind(length, width)
x

# calculate column means
m <- apply(x, 2, mean)
m

# subtract column mean from each column
z <- sweep(x, 2, m)
z

# calculate a covariance matrix
t(z) %*% z / (nrow(z) - 1)


# read rice phenotype data
# this data set was analyzed in Zhao 2011 (Nature Communications 2:467)
pheno <- read.csv("RiceDiversityPheno.csv")

# check the size of data
dim(pheno)

# look at the first 6 lines of data
head(pheno)

# convine the dataset with another dataset
line <- read.csv("RiceDiversityLine.csv")
head(line)

# merge two datasets
data <- merge(line, pheno, by.x = "NSFTV.ID", by.y = "NSFTVID")
head(data)


# calculate ratio
ratio <- data$Seed.length / data$Seed.width
mean(ratio)
ratio

# calculate mean again
mean(ratio, na.rm = T)

# calculate mean of each column
sapply(data, mean, na.rm = T)

# summary of the pheno dataset 
summary(data)


## 2nd step: Visualization
# attach the database to the R search path
attach(data)


## 1d plot
# draw histogram
hist(Plant.height)

# draw stem-and-leaf plot 
stem(Plant.height)

# draw boxplot
boxplot(Plant.height)

# draw histogram again for Blast resistance
hist(Blast.resistance)

# create a contigency table for Blast resistance 
t <- table(Blast.resistance)
t

# draw a barplot for table object
plot(t)

# change the title of axes
plot(t, xlab = "Blast resistance scores", ylab = "Frequency")

# draw another type of barplot
barplot(t)

# draw pie chart
pie(t)

# add the main title
pie(t, main = "Blast resistance")


## 2d plot
# scatter plot
plot(Plant.height, Panicle.length)

# draw regression line
abline(lm(Panicle.length ~ Plant.height))

# draw rug for both axes
rug(Plant.height, side = 1)
rug(Panicle.length, side = 2)

# draw Scatterplot with boxplots showing the marginal distributions
def.par <- par(no.readonly = T)	
layout(matrix(c(2, 0, 1, 3), nrow = 2, byrow = T), widths = c(2, 1), heights = c(1, 2), respect = T)
plot(Plant.height, Panicle.length)
boxplot(Plant.height, horizontal = T)
boxplot(Panicle.length)
par(def.par)

# draw Scatterplot with the bivariate boxplot with the 1d box plots
require(MVA)
x <- cbind(Plant.height, Panicle.length)
x <- na.omit(x)
bvbox(x, xlab = "Plant.height", ylab = "Panicle.length")

# convex hull trimming
hull <- chull(x)
polygon(x[hull, ])
cor(x)
cor(x[-hull, ])


## 3d plot
# draw scatterplot with estimated bivariate density
require("KernSmooth")
x <- cbind(Plant.height, Panicle.length)
x <- na.omit(x)
d <- bkde2D(x, bandwidth = 4)
plot(x)
contour(d$x1, d$x2, d$fhat, add = T)

# draw perspective plot of estimated bivariate density
persp(d$x1, d$x2, d$fhat, xlab = "Plant.height", ylab = "Panicle.length", zlab = "density", theta = -30, phi = 30)

# draw perspective plot using openGL
require("rgl")
persp3d(d$x1, d$x2, d$fhat, xlab = "Plant.height", ylab = "Panicle.length", zlab = "density", col = "green")


## look at the relationship between genetic background and phenotype
pop.id <- as.numeric(Sub.population)
plot(Plant.height, Panicle.length, col = pop.id)
levels(Sub.population)
legend(locator(1), levels(Sub.population), col = 1:nlevels(Sub.population), pch = 1)

# draw boxplot for each sub population
boxplot(Plant.height ~ Sub.population)

# draw Scatterplot with boxplots showing the marginal distributions
def.par <- par(no.readonly = T)	
layout(matrix(c(2, 0, 1, 3), nrow = 2, byrow = T), widths = c(2, 1), heights = c(1, 2), respect = T)
plot(Plant.height, Panicle.length, col = pop.id)
boxplot(Plant.height ~ Sub.population, border = 1:nlevels(Sub.population), horizontal = T)
boxplot(Panicle.length ~ Sub.population, border = 1:nlevels(Sub.population))
par(def.par)

## relationships among more than three variables
# draw bubble plot
symbols(Plant.height, Panicle.length, circles = Flag.leaf.length, inches = 0.1, fg = pop.id)

# draw star plot
x <- data.frame(Plant.height, Panicle.length, Flag.leaf.length)
stars(x)

# draw matrix plot
matplot(t(x), type = "l", lty = 1, col = pop.id)

# draw a scatter plot matrix
pairs(x, col = pop.id)

# draw a bit complex scater plot matrix
pairs(x, panel = function(x, y, ...) {
	points(x, y, ...)
	abline(lm(y ~ x), col = "gray")
}, col = pop.id)

# draw 3d scatter plot using scatterplot3d
require(scatterplot3d)
scatterplot3d(Plant.height, Panicle.length, Flag.leaf.length, color = pop.id)

# draw scatter plot using openGS
plot3d(Plant.height, Panicle.length, Flag.leaf.length, col = pop.id, type = "s", size = 1)

# draw 2d scatter plot with genetic background
plot(Plant.height, Panicle.length, col = pop.id, pch = ".")
stars(cbind(PC1, PC2, PC3, PC4), locations = cbind(Plant.height, Panicle.length), add = T, col.stars = pop.id)
legend(locator(1), levels(Sub.population), col = 1:nlevels(Sub.population), pch = 1)

# draw 2d scatter plot with morphological characteristics
plot(PC1, PC2, col = pop.id, pch = ".")
stars(cbind(Plant.height, Panicle.length, Flag.leaf.length), locations = cbind(PC1, PC2), add = T, col.stars = pop.id, len = 0.005)
legend(locator(1), levels(Sub.population), col = 1:nlevels(Sub.population), pch = 1)

## draw world map and ovarlay origins of germplasm
require(maps)
require(mapdata)
map('worldHires')
points(Longitude, Latitude, col = pop.id)
legend(locator(1), levels(Sub.population), col = 1:nlevels(Sub.population), pch = 1)

map('worldHires')
points(jitter(Longitude, 200), Latitude, col = pop.id)
legend(locator(1), levels(Sub.population), col = 1:nlevels(Sub.population), pch = 1)

# output figure to a PDF file
pdf("map.pdf")
map('worldHires')
points(jitter(Longitude, 200), Latitude, col = pop.id)
legend(-175, 5, levels(Sub.population), col = 1:nlevels(Sub.population), pch = 1)
dev.off()

# output figure to a PDF file
pdf("map_large.pdf", width = 20, height = 10)
map('worldHires')
points(jitter(Longitude, 200), Latitude, col = pop.id)
legend(-175, 5, levels(Sub.population), col = 1:nlevels(Sub.population), pch = 1)
dev.off()
